#pragma once

#include <Colors.hpp>
#include <Engine/types/entity/Entity.h>
#include <Engine/types/entity/mixins/Movable.h>
#include <Engine/types/entity/mixins/Renderable.h>
#include <JGL/types/Texture.h>
#include <J3ML/Geometry/AABB2D.hpp>

namespace Engine {
    class Sprite;
}

using JGL::Texture;
class Engine::Sprite : public Entity, public Renderable, public Movable {
private:
    Texture* texture = nullptr;
    Texture* alpha_mask = nullptr;
protected:
    Vector2 scale = {1, 1};
    // Local space, Where the face_angle rotation should be preformed about.
    // In local space, Where on the texture the entities "position" is.
    // Also, this is where rotations will be done about by default.
    // TODO separate.
    Vector2 origin = {0, 0};
    Color4 base_color = Colors::White;
protected:
    [[nodiscard]] Vector2 GetRenderPosition() const;
    [[nodiscard]] Vector2 GetLocalSpaceOrigin() const;
    [[nodiscard]] virtual Texture* GetTexture() const;
    [[nodiscard]] virtual Texture* GetAlphaMask() const;
protected:
    void SetAlphaMask(Texture* new_alpha_mask);
    void SetOrigin(const Vector2& new_origin);
public:
    void Render() override;
public:
    ~Sprite() override;
    explicit Sprite(const Vector2& pos, const Vector2& origin = {0, 0}, unsigned int layer = 0, const float face_angle = 0.0f, const Color4& base_color = Colors::White, Texture* texture = nullptr,
                    Texture* alpha_mask = nullptr) : Renderable(layer), Movable(pos, face_angle), texture(texture), alpha_mask(alpha_mask), base_color(base_color), origin(origin) {}
};